@@ -1,4 +1,4 @@
-/* internal.h	internal header file for libgsasl
+/* internal.h --- Internal header with hidden library handle structures.
  * Copyright (C) 2002, 2003  Simon Josefsson
  *
  * This file is part of GNU SASL.
@@ -20,68 +20,61 @@
  */
 
 #ifndef _INTERNAL_H
-#define _INTERNAL_H
+# define _INTERNAL_H
 
-#if HAVE_CONFIG_H
-#include "config.h"
-#endif
+# if HAVE_CONFIG_H
+# include "config.h"
+# endif
 
-#ifdef STDC_HEADERS
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <ctype.h>
-#include <string.h>
-#endif
+# ifdef STDC_HEADERS
+#  include <stdio.h>
+#  include <stdlib.h>
+#  include <stdarg.h>
+#  include <ctype.h>
+#  include <string.h>
+# endif
 
 /* Get specifications. */
-#include "gsasl.h"
+# include "gsasl.h"
 
-/* I18n of error codes. */
-#include "gettext.h"
-#define _(String) dgettext (PACKAGE, String)
-#define gettext_noop(String) String
-#define N_(String) gettext_noop (String)
-
-/* Used by *-md5.c. */
-#define HEXCHAR(c) ((c & 0x0F) > 9 ? 'a' + (c & 0x0F) - 10 : '0' + (c & 0x0F))
-
-typedef int (*_Gsasl_code_function) (Gsasl_session_ctx * sctx,
+/* Mechanism function prototypes. */
+typedef int (*_Gsasl_init_function) (Gsasl * ctx);
+typedef void (*_Gsasl_done_function) (Gsasl * ctx);
+typedef int (*_Gsasl_start_function) (Gsasl_session * sctx, void **mech_data);
+typedef int (*_Gsasl_step_function) (Gsasl_session * sctx,
 				     void *mech_data,
-				     const char *input,
-				     size_t input_len,
+				     const char *input, size_t input_len,
+				     char *output, size_t * output_len);
+typedef int (*_Gsasl_finish_function) (Gsasl_session * sctx, void *mech_data);
+typedef int (*_Gsasl_code_function) (Gsasl_session * sctx,
+				     void *mech_data,
+				     const char *input, size_t input_len,
 				     char *output, size_t * output_len);
 
+/* Collection of mechanism functions for either client or server. */
 struct _Gsasl_mechanism_functions
 {
-  int (*init) (Gsasl_ctx * ctx);
-  void (*done) (Gsasl_ctx * ctx);
-  int (*start) (Gsasl_session_ctx * sctx, void **mech_data);
-  int (*step) (Gsasl_session_ctx * sctx,
-	       void *mech_data,
-	       const char *input,
-	       size_t input_len, char *output, size_t * output_len);
-  int (*finish) (Gsasl_session_ctx * sctx, void *mech_data);
+  _Gsasl_init_function init;
+  _Gsasl_done_function done;
+  _Gsasl_start_function start;
+  _Gsasl_step_function step;
+  _Gsasl_finish_function finish;
   _Gsasl_code_function encode;
   _Gsasl_code_function decode;
 };
 
+/* Information about a mechanism. */
 struct _Gsasl_mechanism
 {
   const char *name;
 
-#ifdef USE_CLIENT
   struct _Gsasl_mechanism_functions client;
-#endif
-#ifdef USE_SERVER
   struct _Gsasl_mechanism_functions server;
-#endif
 };
 typedef struct _Gsasl_mechanism _Gsasl_mechanism;
 
-extern _Gsasl_mechanism _gsasl_all_mechanisms[];
-
-struct Gsasl_ctx
+/* Main library handle. */
+struct Gsasl
 {
   size_t n_client_mechs;
   _Gsasl_mechanism *client_mechs;
@@ -113,21 +106,14 @@ struct Gsasl_ctx
   Gsasl_server_callback_gssapi cbs_gssapi;
 };
 
-struct Gsasl_session_ctx
+/* Per-session library handle. */
+struct Gsasl_session
 {
-  Gsasl_ctx *ctx;
+  Gsasl *ctx;
   int clientp;
   _Gsasl_mechanism *mech;
   void *application_data;
   void *mech_data;
 };
 
-#ifndef WITH_STRINGPREP
-extern char *_gsasl_no_stringprep_nfkc (const char *in, ssize_t len);
-extern char *_gsasl_no_stringprep (const char *in, int *stringprep_rc);
-#define gsasl_stringprep_nfkc _gsasl_no_stringprep_nfkc
-#define gsasl_stringprep_saslprep _gsasl_no_stringprep
-#define gsasl_stringprep_trace _gsasl_no_stringprep
-#endif
-
 #endif /* _INTERNAL_H */
