@@ -1,5 +1,5 @@
 /* imap.c --- Implement IMAP profile of SASL login.
- * Copyright (C) 2002, 2003  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004  Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -50,6 +50,9 @@ imap_select_mechanism (char **mechlist)
       /* XXX parse IMAP capability line */
 
       *mechlist = in;
+
+      if (!readln (&in))
+	return 0;
     }
 
   return 1;
@@ -110,6 +113,11 @@ imap_step_recv (char **data)
 
   memmove (&p[0], &p[2], strlen (p) - 1);
 
+  if (p[strlen(p)-1] == '\n')
+    p[strlen(p)-1] = '\0';
+  if (p[strlen(p)-1] == '\r')
+    p[strlen(p)-1] = '\0';
+
   return 1;
 }
 
