@@ -68,197 +68,110 @@ struct sasltv
   char *suggestpin;
   char *pin;
   int securidrc;
-}
-sasltv[] =
-{
+};
+struct sasltv sasltv[] = {
 #ifdef USE_EXTERNAL
-  {
-    CLIENT, "EXTERNAL",
-    {
-    "", NULL}
-  }
-  ,
-  {
-    SERVER, "EXTERNAL",
-    {
-    "", NULL}
-  }
-  ,
+  {CLIENT, "EXTERNAL", {"", NULL}},
+  {SERVER, "EXTERNAL", {"", NULL}},
 #endif
 #ifdef USE_ANONYMOUS
-  {
-    CLIENT, "ANONYMOUS",
-    {
-    "", "Zm9vQGJhci5jb20=", NULL, NULL}
-  , NULL, NULL, NULL, NULL, NULL, NULL, "foo@bar.com"}
-  ,
-  {
-    SERVER, "ANONYMOUS",
-    {
-    "Zm9vQGJhci5jb20=", NULL, NULL}
-  , NULL, NULL, NULL, NULL, NULL, NULL, "foo@bar.com"}
-  ,
+  {CLIENT, "ANONYMOUS", {"", "Zm9vQGJhci5jb20=", NULL, NULL}, NULL, NULL,
+   NULL, NULL, NULL, NULL, "foo@bar.com"},
+  {SERVER, "ANONYMOUS", {"Zm9vQGJhci5jb20=", NULL, NULL}, NULL, NULL, NULL,
+   NULL, NULL, NULL, "foo@bar.com"},
 #endif
 #ifdef USE_NTLM
-  {
-    CLIENT, "NTLM",
-    {
-    "Kw==",
-	"TlRMTVNTUAABAAAAB7IAAAcABwAgAAAAAAAAAAcAAABhdXRoemlk",
-	"TlRMTVNTUAAAAAAAAAAAAAAAAAAAAGFiY2RlZmdoMDEyMzQ1Njc4ODY2NDQwMTIz",
-	"TlRMTVNTUAADAAAAGAAYAEAAAAAYABgAWAAAAAAAAAAwAAAADgAOAHAAAAAOAA4AfgAAAAAAAABMAAAAAABhYheBBp9xJad9eYo3oh1k55GNFDIui8H8Qz4CfWYVVToBhVzFFbzyzqAZN5Wl59K/FmEAdQB0AGgAegBpAGQAYQB1AHQAaAB6AGkAZAA=",
-	NULL, NULL}
-  , "password", "authzid", "authid"}
-  ,
+  {CLIENT, "NTLM",
+   {"Kw==", "TlRMTVNTUAABAAAAB7IAAAcABwAgAAAAAAAAAAcAAABhdXRoemlk",
+    "TlRMTVNTUAAAAAAAAAAAAAAAAAAAAGFiY2RlZmdoMDEyMzQ1Njc4ODY2NDQwMTIz",
+    "TlRMTVNTUAADAAAAGAAYAEAAAAAYABgAWAAAAAAAAAAwAAAADgAOAHAAAAAOAA4AfgAAA"
+    "AAAAABMAAAAAABhYheBBp9xJad9eYo3oh1k55GNFDIui8H8Qz4CfWYVVToBhVzFFbzyzq"
+    "AZN5Wl59K/FmEAdQB0AGgAegBpAGQAYQB1AHQAaAB6AGkAZAA=",
+    NULL, NULL}, "password", "authzid", "authid"},
 #endif
 #ifdef USE_PLAIN
-  {
-    CLIENT, "PLAIN",
-    {
-    "", "YXV0aHppZABhdXRoaWQAcGFzc3dvcmQ=", NULL, NULL}
-  , "password", "authzid", "authid"}
-  ,
-  {
-    CLIENT, "PLAIN",
-    {
-    "", "YQBhAGE=", NULL, NULL}
-  , "a", "a", "a"}
-  ,
+  {CLIENT, "PLAIN",
+   {"", "YXV0aHppZABhdXRoaWQAcGFzc3dvcmQ=", NULL, NULL}, "password",
+   "authzid", "authid"},
+  {CLIENT, "PLAIN",
+   {"", "YQBhAGE=", NULL, NULL}, "a", "a", "a"},
 #ifdef WITH_STRINGPREP
-  {
-    CLIENT, "PLAIN",
-    {
-    "", "YQBhAGE=", NULL, NULL}
-  , UTF8_a, UTF8_a, UTF8_a}
-  ,
+  {CLIENT, "PLAIN",
+   {"", "YQBhAGE=", NULL, NULL}, UTF8_a, UTF8_a, UTF8_a},
 #endif
-  {
-    SERVER, "PLAIN",
-    {
-    "YXV0aHppZABhdXRoaWQAcGFzc3dvcmQ=", NULL, NULL}
-  , "password", "authzid", "authid"}
-  ,
-  {
-    SERVER, "PLAIN",
-    {
-    "", "", "YXV0aHppZABhdXRoaWQAcGFzc3dvcmQ=", NULL, NULL}
-  , "password", "authzid", "authid"}
-  ,
+  {SERVER, "PLAIN",
+   {"YXV0aHppZABhdXRoaWQAcGFzc3dvcmQ=", NULL, NULL}, "password", "authzid",
+   "authid"},
+  {SERVER, "PLAIN",
+   {"", "", "YXV0aHppZABhdXRoaWQAcGFzc3dvcmQ=", NULL, NULL}, "password",
+   "authzid", "authid"},
 #endif
 #ifdef USE_LOGIN
-  {
-    CLIENT, "LOGIN",
-    {
-    "VXNlciBOYW1l", "YXV0aHppZA==", "UGFzc3dvcmQ=", "cGFzc3dvcmQ=", NULL,
-	NULL}
-  , "password", "authzid", "authid"}
-  ,
-  {
-    CLIENT, "LOGIN",
-    {
-    "VXNlciBOYW1l", "YQ==", "UGFzc3dvcmQ=", "YQ==", NULL, NULL}
-  , "a", "a", "authid"}
-  ,
+  {CLIENT, "LOGIN",
+   {"VXNlciBOYW1l", "YXV0aHppZA==", "UGFzc3dvcmQ=", "cGFzc3dvcmQ=", NULL,
+    NULL}, "password", "authzid", "authid"},
+  {CLIENT, "LOGIN",
+   {"VXNlciBOYW1l", "YQ==", "UGFzc3dvcmQ=", "YQ==", NULL, NULL}, "a", "a",
+   "authid"},
 #ifdef WITH_STRINGPREP
-  {
-    CLIENT, "LOGIN",
-    {
-    "VXNlciBOYW1l", "YQ==", "UGFzc3dvcmQ=", "YQ==", NULL, NULL}
-  , UTF8_a, UTF8_a, "authid"}
-  ,
+  {CLIENT, "LOGIN",
+   {"VXNlciBOYW1l", "YQ==", "UGFzc3dvcmQ=", "YQ==", NULL, NULL}, UTF8_a,
+   UTF8_a, "authid"},
 #endif
-  {
-    SERVER, "LOGIN",
-    {
-    "", "VXNlciBOYW1l", "YXV0aHppZA==", "UGFzc3dvcmQ=", "cGFzc3dvcmQ=",
-	NULL, NULL}
-  , "password", "authzid", "authid"}
-  ,
+  {SERVER, "LOGIN",
+   {"", "VXNlciBOYW1l", "YXV0aHppZA==", "UGFzc3dvcmQ=", "cGFzc3dvcmQ=",
+    NULL, NULL}, "password", "authzid", "authid"},
 #endif
 #ifdef USE_CRAM_MD5
-  {
-    CLIENT, "CRAM-MD5",
-    {
-    "PGNiNmQ5YTQ5ZDA3ZjEwY2MubGliZ3Nhc2xAbG9jYWxob3N0Pg==",
-	"YXV0aGlkIGZkNjRmMjYxZWYxYjBjYjg0ZmZjNGVmYzgwZDk3NjFj", NULL, NULL}
-  , "password", "authzid", "authid"}
-  ,
+  {CLIENT, "CRAM-MD5",
+   {"PGNiNmQ5YTQ5ZDA3ZjEwY2MubGliZ3Nhc2xAbG9jYWxob3N0Pg==",
+    "YXV0aGlkIGZkNjRmMjYxZWYxYjBjYjg0ZmZjNGVmYzgwZDk3NjFj", NULL, NULL},
+   "password", "authzid", "authid"},
 #endif
 #ifdef USE_SECURID
-  {
-    CLIENT, "SECURID",
-    {
-    "", "YXV0aHppZABhdXRoaWQANDcxMQA=", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711"}
-  ,
-  {
-    CLIENT, "SECURID",
-    {
-    "", "YXV0aHppZABhdXRoaWQANDcxMQA=",
-	"cGFzc2NvZGU=", "YXV0aHppZABhdXRoaWQANDcxMQA=", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711"}
-  ,
-  {
-    CLIENT, "SECURID",
-    {
-    "", "YXV0aHppZABhdXRoaWQANDcxMQA=",
-	"cGlu", "YXV0aHppZABhdXRoaWQANDcxMQA0MgA=", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711", NULL, "42"}
-  ,
-  {
-    CLIENT, "SECURID",
-    {
-    "", "YXV0aHppZABhdXRoaWQANDcxMQA=",
-	"cGluMjM=", "YXV0aHppZABhdXRoaWQANDcxMQA0MgA=", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711", "23", "42"}
-  ,
-  {
-    CLIENT, "SECURID",
-    {
-    "", "YXV0aHppZABhdXRoaWQANDcxMQA=",
-	"cGluMjM=", "YXV0aHppZABhdXRoaWQANDcxMQA0MgA=",
-	"cGFzc2NvZGU=", "YXV0aHppZABhdXRoaWQANDcxMQA=", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711", "23", "42"}
-  ,
-  {
-    SERVER, "SECURID",
-    {
-    "YXV0aHppZABhdXRoaWQANDcxMQA=", "", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711"}
-  ,
-  {
-    SERVER, "SECURID",
-    {
-    "YXV0aHppZABhdXRoaWQANDcxMQA=", "", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711"}
-  ,
-  {
-    SERVER, "SECURID",
-    {
-    "YXV0aHppZABhdXRoaWQANDcxMQA=", "cGlu",
-	"YXV0aHppZABhdXRoaWQANDcxMQA0MgA=", "", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711", NULL, "42",
-      GSASL_SECURID_SERVER_NEED_NEW_PIN}
-  ,
-  {
-    SERVER, "SECURID",
-    {
-    "YXV0aHppZABhdXRoaWQANDcxMQA=", "cGluMTc=",
-	"YXV0aHppZABhdXRoaWQANDcxMQAyMwA=", "", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711", "17", "23",
-      GSASL_SECURID_SERVER_NEED_NEW_PIN}
-  ,
-  {
-    SERVER, "SECURID",
-    {
-    "YXV0aHppZABhdXRoaWQANDcxMQA=", "cGFzc2NvZGU=",
-	"YXV0aHppZABhdXRoaWQANDcxMQA=", NULL, NULL}
-  , NULL, "authzid", "authid", NULL, NULL, NULL, NULL, "4711", NULL, NULL,
-      GSASL_SECURID_SERVER_NEED_ADDITIONAL_PASSCODE}
-  ,
+  {CLIENT, "SECURID",
+   {"", "YXV0aHppZABhdXRoaWQANDcxMQA=", NULL, NULL}, NULL, "authzid",
+   "authid", NULL, NULL, NULL, NULL, "4711"},
+  {CLIENT, "SECURID",
+   {"", "YXV0aHppZABhdXRoaWQANDcxMQA=", "cGFzc2NvZGU=",
+    "YXV0aHppZABhdXRoaWQANDcxMQA=", NULL, NULL}, NULL, "authzid", "authid",
+   NULL, NULL, NULL, NULL, "4711"},
+  {CLIENT, "SECURID",
+   {"", "YXV0aHppZABhdXRoaWQANDcxMQA=", "cGlu",
+    "YXV0aHppZABhdXRoaWQANDcxMQA0MgA=", NULL, NULL}, NULL, "authzid",
+   "authid", NULL, NULL, NULL, NULL, "4711", NULL, "42"},
+  {CLIENT, "SECURID",
+   {"", "YXV0aHppZABhdXRoaWQANDcxMQA=", "cGluMjM=",
+    "YXV0aHppZABhdXRoaWQANDcxMQA0MgA=", NULL, NULL}, NULL, "authzid",
+   "authid", NULL, NULL, NULL, NULL, "4711", "23", "42"},
+  {CLIENT, "SECURID",
+   {"", "YXV0aHppZABhdXRoaWQANDcxMQA=", "cGluMjM=",
+    "YXV0aHppZABhdXRoaWQANDcxMQA0MgA=", "cGFzc2NvZGU=",
+    "YXV0aHppZABhdXRoaWQANDcxMQA=", NULL, NULL}, NULL, "authzid", "authid",
+   NULL, NULL, NULL, NULL, "4711", "23", "42"},
+  {SERVER, "SECURID",
+   {"YXV0aHppZABhdXRoaWQANDcxMQA=", "", NULL, NULL}, NULL, "authzid",
+   "authid", NULL, NULL, NULL, NULL, "4711"},
+  {SERVER, "SECURID",
+   {"YXV0aHppZABhdXRoaWQANDcxMQA=", "", NULL, NULL}, NULL, "authzid",
+   "authid", NULL, NULL, NULL, NULL, "4711"},
+  {SERVER, "SECURID",
+   {"YXV0aHppZABhdXRoaWQANDcxMQA=", "cGlu",
+    "YXV0aHppZABhdXRoaWQANDcxMQA0MgA=", "", NULL, NULL}, NULL, "authzid",
+   "authid", NULL, NULL, NULL, NULL, "4711", NULL, "42",
+   GSASL_SECURID_SERVER_NEED_NEW_PIN},
+  {SERVER, "SECURID",
+   {"YXV0aHppZABhdXRoaWQANDcxMQA=", "cGluMTc=",
+    "YXV0aHppZABhdXRoaWQANDcxMQAyMwA=", "", NULL, NULL}, NULL, "authzid",
+   "authid", NULL, NULL, NULL, NULL, "4711", "17", "23",
+   GSASL_SECURID_SERVER_NEED_NEW_PIN},
+  {SERVER, "SECURID",
+   {"YXV0aHppZABhdXRoaWQANDcxMQA=", "cGFzc2NvZGU=",
+    "YXV0aHppZABhdXRoaWQANDcxMQA=", NULL, NULL}, NULL, "authzid", "authid",
+   NULL, NULL, NULL, NULL, "4711", NULL, NULL,
+   GSASL_SECURID_SERVER_NEED_ADDITIONAL_PASSCODE},
 #endif
-  {
-  -1}
+  {-1}
 };
 
 static int
@@ -267,10 +180,14 @@ client_callback_authorization_id (Gsasl_session_ctx * xctx,
 {
   Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
   int i = *(int *) gsasl_application_data_get (ctx);
+  size_t needlen = strlen (sasltv[i].authzid);
 
-  *outlen = strlen (sasltv[i].authzid);
+  if (*outlen < needlen)
+    return GSASL_TOO_SMALL_BUFFER;
+
+  *outlen = needlen;
   if (out)
-    memcpy (out, sasltv[i].authzid, strlen (sasltv[i].authzid));
+    memcpy (out, sasltv[i].authzid, needlen);
 
   return GSASL_OK;
 }
@@ -281,10 +198,14 @@ client_callback_authentication_id (Gsasl_session_ctx * xctx,
 {
   Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
   int i = *(int *) gsasl_application_data_get (ctx);
+  size_t needlen = strlen (sasltv[i].authid);
 
-  *outlen = strlen (sasltv[i].authid);
+  if (*outlen < needlen)
+    return GSASL_TOO_SMALL_BUFFER;
+
+  *outlen = needlen;
   if (out)
-    memcpy (out, sasltv[i].authid, strlen (sasltv[i].authid));
+    memcpy (out, sasltv[i].authid, needlen);
 
   return GSASL_OK;
 }
@@ -295,10 +216,14 @@ client_callback_password (Gsasl_session_ctx * xctx, char *out,
 {
   Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
   int i = *(int *) gsasl_application_data_get (ctx);
+  size_t needlen = strlen (sasltv[i].password);
 
-  *outlen = strlen (sasltv[i].password);
+  if (*outlen < needlen)
+    return GSASL_TOO_SMALL_BUFFER;
+
+  *outlen = needlen;
   if (out)
-    memcpy (out, sasltv[i].password, strlen (sasltv[i].password));
+    memcpy (out, sasltv[i].password, needlen);
 
   return GSASL_OK;
 }
@@ -337,10 +262,14 @@ server_callback_retrieve (Gsasl_session_ctx * xctx,
 {
   Gsasl_ctx *ctx = gsasl_server_ctx_get (xctx);
   int i = *(int *) gsasl_application_data_get (ctx);
+  size_t needlen = strlen (sasltv[i].password);
+
+  if (*keylen < needlen)
+    return GSASL_TOO_SMALL_BUFFER;
 
   *keylen = strlen (sasltv[i].password);
   if (key)
-    memcpy (key, sasltv[i].password, strlen (sasltv[i].password));
+    memcpy (key, sasltv[i].password, needlen);
 
   return GSASL_OK;
 }
@@ -361,8 +290,12 @@ client_callback_anonymous (Gsasl_session_ctx * xctx, char *out,
 {
   Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
   int i = *(int *) gsasl_application_data_get (ctx);
+  size_t needlen = strlen (sasltv[i].anonymous);
 
-  *outlen = strlen (sasltv[i].anonymous);
+  if (*outlen < needlen)
+    return GSASL_TOO_SMALL_BUFFER;
+
+  *outlen = needlen;
   if (out)
     memcpy (out, sasltv[i].anonymous, strlen (sasltv[i].anonymous));
 
@@ -391,10 +324,14 @@ client_callback_passcode (Gsasl_session_ctx * xctx, char *out,
 {
   Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
   int i = *(int *) gsasl_application_data_get (ctx);
+  size_t needlen = strlen (sasltv[i].passcode);
 
-  *outlen = strlen (sasltv[i].passcode);
+  if (*outlen < needlen)
+    return GSASL_TOO_SMALL_BUFFER;
+
+  *outlen = needlen;
   if (out)
-    memcpy (out, sasltv[i].passcode, strlen (sasltv[i].passcode));
+    memcpy (out, sasltv[i].passcode, needlen);
 
   return GSASL_OK;
 }
@@ -405,6 +342,7 @@ client_callback_pin (Gsasl_session_ctx * xctx, char *suggestion,
 {
   Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
   int i = *(int *) gsasl_application_data_get (ctx);
+  size_t needlen = strlen (sasltv[i].pin);
 
   if (suggestion && sasltv[i].suggestpin &&
       strcmp (suggestion, sasltv[i].suggestpin) != 0)
@@ -414,12 +352,12 @@ client_callback_pin (Gsasl_session_ctx * xctx, char *suggestion,
       (suggestion != NULL && sasltv[i].suggestpin == NULL))
     return GSASL_AUTHENTICATION_ERROR;
 
-  if (*outlen < strlen (sasltv[i].pin))
+  if (*outlen < needlen)
     return GSASL_TOO_SMALL_BUFFER;
 
-  *outlen = strlen (sasltv[i].pin);
+  *outlen = needlen;
   if (out)
-    memcpy (out, sasltv[i].pin, strlen (sasltv[i].pin));
+    memcpy (out, sasltv[i].pin, needlen);
 
   return GSASL_OK;
 }
